/**
 * 	\file		TxDuino.h
 *  \author:    Joshua J Bialkowski (jbialk@mit.edu)
 *  \date:      Oct 27, 2009
 *  \brief:
 *
 *  detail:
 */

#ifndef CTXDUINO_H_
#define CTXDUINO_H_


#include <vector>
#include <string>

#include "types.h"


namespace txduino
{


/**
 *  \brief  operating system dependendant implementation structure
 */
struct STxDuinoImpl;

/**
 *  \brief Class encapsulating the interface / API for communicating with one
 *  TxDuino device. The TxDuino sends a standard RC PPM signal encoding
 *  commands for up to 8 PWM channels (servos, engine controllers).
 *
 *  The magnitude of each channel is divided into 250 discrete segments.
 *  Exactly how those segments are interpreted by the actuators is determined
 *  by the configuration of this object.
 *
 *  note: channels are zero indexed
 *
 *  \todo   add a constructor that accepts a csv file with the saturations and
 *          neutral points that can be generated from the test program
 */
class CTxDuino
{
    private:
        STxDuinoImpl*   m_osInfo;   /// pointer to os-dependant implementation
        std::string     m_devName;  /// system device name,
                                    /// i.e. "\\.\COM2", "/dev/tty2"

        u8          m_chan      [9];    /// value for each channel [0,250]
        u8          m_neutral   [8];    /// the "center" for each actuator
        u8          m_minsat    [8];    /// the minimum value for each channel
        u8          m_maxsat    [8];    /// the maximum value for each channel


    public:
        /**
         *  \brief  Constructs a new TxDuino object serving as an interface
         *          into on particular TxDuino device.
         *  \param  strDevice   device string, i.e. "\\.\COM2", "/dev/tty2"
         */
        CTxDuino( std::string strDevice );


        /**
         *  \brief  cleans up OS resources reserved for this serial connection
         */
        virtual ~CTxDuino();


        /**
         *  \brief  sends the current channel definitions to the device
         */
        void send();


        /**
         *  \brief  sets the value of an actuator as a percent of it's viable
         *          range.
         *  \param  chan        the channel to set
         *  \param  percent     -100% < percent < 100%; value to set channel to
         */
        void setPercent( s32 chan, f64 percent );


        /**
         *  \brief  returns the percent value the indicated actuator is set to,
         *          note: if the neutral point is equal to one of the saturation
         *          points this value may be unreliable
         *  \param  chan        the channel to get
         *  \return percent value of actuator on indicated channel
         */
        f64 getPercent( s32 chan );


        /**
         *  \brief  sets the raw value of the actuator pulse width
         *  \param  chan        the channel to set
         *  \param  value      0 < value < 250; value to set channel to
         */
        void setRaw( s32 chan, u8 value );


        /**
         *  \brief  returns the raw value the indicated actuator is set to
         *  \param  chan        the channel to get
         *  \return a value between 0 and 250 indicated the pulse length for
         *          that actuator (multiply by 4us and add 700us to get the
         *          actual pulse length)
         */
        u8 getRaw( s32 chan );




        /**
         *  \brief  sets the raw value of the actuator pulse width corresponding
         *          to a neutral state of that actuator
         *  \param  chan        the channel to set
         *  \param  value      0 < value < 250; value to set channel to
         */
        void setNeutral( s32 chan, u8 value );


        /**
         *  \brief  returns the raw value corresponding to a neutral state of
         *          the indicated actuator
         *  \param  chan        the channel to get
         *  \return a value between 0 and 250 indicated the pulse length for
         *          that actuator (multiply by 4us and add 700us to get the
         *          actual pulse length)
         */
        u8 getNeutral( s32 chan );




        /**
         *  \brief  sets the raw value of the actuator pulse width corresponding
         *          to the minimum state of the indicated actuator
         *  \param  chan        the channel to set
         *  \param  value      0 < value < 250; value to set channel to
         */
        void setMinSat( s32 chan, u8 value );


        /**
         *  \brief  returns the raw value corresponding to a minimum state of
         *          the indicated actuator
         *  \param  chan        the channel to get
         *  \return a value between 0 and 250 indicated the pulse length for
         *          that actuator (multiply by 4us and add 700us to get the
         *          actual pulse length)
         */
        u8 getMinSat( s32 chan );




        /**
         *  \brief  sets the raw value of the actuator pulse width corresponding
         *          to the maximum state of the indicated actuator
         *  \param  chan        the channel to set
         *  \param  value      0 < value < 250; value to set channel to
         */
        void setMaxSat( s32 chan, u8 value );


        /**
         *  \brief  returns the raw value corresponding to a maximum state of
         *          the indicated actuator
         *  \param  chan        the channel to get
         *  \return a value between 0 and 250 indicated the pulse length for
         *          that actuator (multiply by 4us and add 700us to get the
         *          actual pulse length)
         */
        u8 getMaxSat( s32 chan );


        /**
         *  \brief  return the device name that was used to connect to this
         *          txduino
         */
        std::string getName();

};

}

#endif /* CTXDUINO_H_ */

